## Sliced rotated sphere packing designs
## Last modified on May 4, 2017
## 
## Author:   Blinded
## Contact:  Blinded
## Website:  Blinded
##
## RSPD_Am(n,m)$design  gives an A_m-based rotated sphere packing design.
##
## RSPD_AmAms(n,m)$candidates  gives an A_m-A_m^*-based sliced rotated sphere packing design by enlarging an A_m-based rotated sphere packing design.
##   Its first m columns give the design points; rows whose (2*m+1)th column being zero are from the A_m-based design and other rows are from other slices.
##
## SRSPD(n,m)  gives an A_m-A_m^*-based sliced rotated sphere packing design by partitioning an A_m^*-based rotated sphere packing design, 
##   where SRSPD(n,m)$design gives the design points and SRSPD(n,m)$slices gives the slice index of the points. 



mod = function(a,b) { return(a-b*floor(a/b)) }

fgr2 = function(D) {
	dis = (dist(D[, 1]))^2
	for (j in 2:dim(D)[2]) dis = dis * (dist(D[, j]))^2
	DIST = as.matrix(dis)
	fn = sum(1/dis)
	lfn = log(fn)
	return(lfn)
}
  
addAm <- function(ballr,mm) {
	if(mm==1) FE <- (-floor(ballr)):floor(ballr)   
	if(mm>1) { 
		FE = cbind(0, addAm(sqrt(ballr^2),mm-1));
		if(floor(ballr)>0) for(s in 1:floor(ballr)) { FEE <- addAm(sqrt(ballr^2-s^2),mm-1); FE <- rbind( FE, cbind(rep(s,length(FEE)/(mm-1)),FEE) ); }
		if(floor(ballr)>0) for(s in (-1):(-floor(ballr))) { FEE <- addAm(sqrt(ballr^2-s^2),mm-1); FE <- rbind( FE, cbind(rep(s,length(FEE)/(mm-1)),FEE) ); } 
	}
	return(FE)
}

  
addfrom <- function(m,tryF,ballr2) {
	sumF <- sum(tryF) 
	if( sum(tryF^2)*(m+1)/m - sumF^2/m > ballr2 ) return(0)
	dd2m <- rep(0,2*m+2)
	dd2m[1:m] <- -2*(m+1)*tryF +2*sumF +m
	dd2m[(m+1):(2*m)] <- 2*(m+1)*tryF -2*sumF +m
	dd2m[2*m+1] <- -2*sumF +m
	dd2m[2*m+2] <- 2*sumF +m
	minv=0
	mind=0
	for(s in 1:(2*m+2)) if(dd2m[s]<minv) { minv <- dd2m[s]; mind <- s; }
	return(mind)
}


RSPD_Am <- function(n, m=2, w=100){

	n <- as.integer(n)
	if(is.na(n)) stop("n must be an integer greater than one")
	if(n<=1) stop("n must be an integer greater than one")
	m <- as.integer(m)
	if(is.na(m)) stop("m must be an integer greater than one")
	if(m<=1) stop("m must be an integer greater than one")
	w <- as.integer(w)
	if(is.na(w)) stop("w must be an integer greater than one")
	if(w<=0) stop("w must be an positive integer")

	G <- matrix(-(sqrt(m+1)+1)/sqrt(2)/m,m,m)
	diag(G) <- diag(G) + sqrt(2)/2
	detG <- abs(det(G))
	intm <- floor((m+1)/2)
	rhoc <- 1/2 * sqrt(2*intm*(m+1-intm)/(m+1))
	Omega <- pi^(m/2)/gamma(m/2+1)
	Theta <- Omega*rhoc^m/detG
	l <- (n*detG)^(1/m)
	
	nsd=0
	for(r in (m+1)/m^seq(1.7,.99,-0.05))
	{
		ballr <- l/2 * sqrt(m) + rhoc*r 
		FE <- addAm(ballr*sqrt(2),m)
		FEdis <- rep(0,dim(FE)[1])
		for(i in 1:dim(FE)[1]) { FEdis[i] <- sum(FE[i,])^2/2 + sum(FE[i,]^2)/2; } 
		FE <- FE[ FEdis <= ballr^2, ]
		
		minfgr2=0  
		for(ll in 1:(w*2)) {
			
			CPair <- matrix(0,m*(m-1)/2,2)
			row <- 1
			for(i in 1:(m-1)) for(j in (i+1):m) { CPair[row,1] <- i; CPair[row,2] <- j; row <- row+1; }
			R <- diag(m)
			if(m>2) for(a in 1:((m*(m-1)/2))) {
				alpha <- runif(1,0,2*pi)
				thepair <- CPair[a,]
				W <- diag(m)
				W[thepair[1],thepair[1]] <- cos(alpha); W[thepair[2],thepair[2]] <- cos(alpha); 
				W[thepair[1],thepair[2]] <- sin(alpha); W[thepair[2],thepair[1]] <- -sin(alpha); 
				R <- R%*%W
			}
			GR <- G %*% R
			E <- FE %*% GR 

			isE <- FALSE
			isB <- FALSE
			isL <- FALSE
			epsilonV <- matrix(0,3,m)
			for(i in 1:20) {
				epsilon <- runif(m,-rhoc*r,rhoc*r)
				D <- E
				for(j in 1:m) { if(length(D)<=m) break; D <- D[ abs(D[,j]-epsilon[j])<l/2 ,]; }
				if(length(D)<=m) next; 
				if(dim(D)[1]==n) { isE <- TRUE; epsilonV[1,] <- epsilon; break; }
				if(isL==FALSE & dim(D)[1]<n) { isL <- TRUE; epsilonV[2,] <- epsilon; }
				if(isB==FALSE & dim(D)[1]>n) { isB <- TRUE; epsilonV[3,] <- epsilon; }
				if(isL==TRUE & isB==TRUE) break;
			}
			try1 <- i
			if(try1==20) next; 
			try2 <- 0
			try3 <- 0
			
			if(isE==FALSE) for(j in 1:(m-1)) {
				try2 <- j
				epsilon <- c(epsilonV[2,1:j],epsilonV[3,(j+1):m])
				D <- E
				for(j in 1:m) { if(length(D)<=m) break;  D <- D[ abs(D[,j]-epsilon[j])<l/2 ,]; }
				if(length(D)<=m) next; 
				if(dim(D)[1]==n) { isE <- TRUE; epsilonV[1,] <- epsilon; break; }
				if(dim(D)[1]<n) { epsilonV[2,] <- epsilon; break; }
				if(dim(D)[1]>n) { epsilonV[3,] <- epsilon; }
			}

			if(isE==FALSE) for(k in 1:20) { 
				try3 <- k
				epsilon <- (epsilonV[2,]+epsilonV[3,])/2
				D <- E
				for(j in 1:m) { if(length(D)<=m) break; D <- D[ abs(D[,j]-epsilon[j])<l/2 ,]; }
				if(length(D)<=m) next; 
				if(dim(D)[1]==n) { isE <- TRUE; epsilonV[1,] <- epsilon; break; }
				if(dim(D)[1]<n) { epsilonV[2,] <- epsilon; }
				if(dim(D)[1]>n) { epsilonV[3,] <- epsilon; }
			}
			if(try3==20) next;
			
			nsd=nsd+1
			for(j in 1:m) D[,j] <- (D[,j]-epsilon[j]) / l + .5

			fgr2D=fgr2(D)
			if(nsd==1|fgr2D<minfgr2) {
				Dopt <- D
				minfgr2 <- fgr2D
				Ropt <- R
				epsilonopt <- epsilon
			}
			
			if(nsd>=w) break; 
		}
		
		if(nsd>=w) break; 
	}
		
	D=Dopt

	list(design=Dopt,generator=G,rotation=Ropt,delta=-epsilonopt,Theta=Theta,l=l,worstdistance=rhoc/l,minfgr2=minfgr2)
}



RSPD_AmAms <- function(n, m=2, w=100){

	n <- as.integer(n)
	if(is.na(n)) stop("n must be an integer greater than one")
	if(n<=1) stop("n must be an integer greater than one")
	m <- as.integer(m)
	if(is.na(m)) stop("m must be an integer greater than one")
	if(m<=1) stop("m must be an integer greater than one")
	w <- as.integer(w)
	if(is.na(w)) stop("w must be an integer greater than one")
	if(w<=0) stop("w must be an positive integer")

	G <- matrix(-(sqrt(m+1)+1)/sqrt(2)/m,m,m)
	diag(G) <- diag(G) + sqrt(2)/2
	detG <- abs(det(G))
	intm <- floor((m+1)/2)
	rhoc <- 1/2 * sqrt(2*intm*(m+1-intm)/(m+1))
	Omega <- pi^(m/2)/gamma(m/2+1)
	Theta <- Omega*rhoc^m/detG
	l <- (n*detG)^(1/m)
	
	nsd=0
	for(r in (m+1)/m^seq(1.7,.99,-0.05))
	{
		ballr <- l/2 * sqrt(m) + rhoc*r 
		FE <- addAm(ballr*sqrt(2),m)
		FEdis <- rep(0,dim(FE)[1])
		for(i in 1:dim(FE)[1]) { FEdis[i] <- sum(FE[i,])^2/2 + sum(FE[i,]^2)/2; } 
		FE <- FE[ FEdis <= ballr^2, ]
		
		minfgr2=0  
		for(ll in 1:(w*2)) {
			
			CPair <- matrix(0,m*(m-1)/2,2)
			row <- 1
			for(i in 1:(m-1)) for(j in (i+1):m) { CPair[row,1] <- i; CPair[row,2] <- j; row <- row+1; }
			R <- diag(m)
#			if(m>2) 
			for(a in 1:((m*(m-1)/2))) {
				alpha <- runif(1,0,2*pi)
				thepair <- CPair[a,]
				W <- diag(m)
				W[thepair[1],thepair[1]] <- cos(alpha); W[thepair[2],thepair[2]] <- cos(alpha); 
				W[thepair[1],thepair[2]] <- sin(alpha); W[thepair[2],thepair[1]] <- -sin(alpha); 
				R <- R%*%W
			}
			GR <- G %*% R
			E <- FE %*% GR 

			isE <- FALSE
			isB <- FALSE
			isL <- FALSE
			epsilonV <- matrix(0,3,m)
			for(i in 1:20) {
				epsilon <- runif(m,-rhoc*r,rhoc*r)
				D <- E
				for(j in 1:m) { if(length(D)<=m) break; D <- D[ abs(D[,j]-epsilon[j])<l/2 ,]; }
				if(length(D)<=m) next; 
				if(dim(D)[1]==n) { isE <- TRUE; epsilonV[1,] <- epsilon; break; }
				if(isL==FALSE & dim(D)[1]<n) { isL <- TRUE; epsilonV[2,] <- epsilon; }
				if(isB==FALSE & dim(D)[1]>n) { isB <- TRUE; epsilonV[3,] <- epsilon; }
				if(isL==TRUE & isB==TRUE) break;
			}
			try1 <- i
			if(try1==20) next; 
			try2 <- 0
			try3 <- 0
			
			if(isE==FALSE) for(j in 1:(m-1)) {
				try2 <- j
				epsilon <- c(epsilonV[2,1:j],epsilonV[3,(j+1):m])
				D <- E
				for(j in 1:m) { if(length(D)<=m) break;  D <- D[ abs(D[,j]-epsilon[j])<l/2 ,]; }
				if(length(D)<=m) next; 
				if(dim(D)[1]==n) { isE <- TRUE; epsilonV[1,] <- epsilon; break; }
				if(dim(D)[1]<n) { epsilonV[2,] <- epsilon; break; }
				if(dim(D)[1]>n) { epsilonV[3,] <- epsilon; }
			}

			if(isE==FALSE) for(k in 1:20) { 
				try3 <- k
				epsilon <- (epsilonV[2,]+epsilonV[3,])/2
				D <- E
				for(j in 1:m) { if(length(D)<=m) break; D <- D[ abs(D[,j]-epsilon[j])<l/2 ,]; }
				if(length(D)<=m) next; 
				if(dim(D)[1]==n) { isE <- TRUE; epsilonV[1,] <- epsilon; break; }
				if(dim(D)[1]<n) { epsilonV[2,] <- epsilon; }
				if(dim(D)[1]>n) { epsilonV[3,] <- epsilon; }
			}
			if(try3==20) next;
			
			nsd=nsd+1
			for(j in 1:m) D[,j] <- (D[,j]-epsilon[j]) / l + .5

			fgr2D=fgr2(D)
			if(nsd==1|fgr2D<minfgr2) {
				Dopt <- D
				minfgr2 <- fgr2D
				Ropt <- R
				epsilonopt <- epsilon
			}
			
			if(nsd>=w) break; 
		}
		
		if(nsd>=w) break; 
	}
		
	D0=Dopt
	R=Ropt
#	epsilon=epsilonopt*(m+1)^(1/m)
	epsilon=epsilonopt*sqrt(2+2/m)

	n1=n*(m+1)
	
	G <- matrix(-1,m,m)
	diag(G) <- m-sqrt(m+1)
	G <- G/(sqrt(m+1)-1)/sqrt(m)
	etamin <- sqrt(m+1)/sqrt(2*m)
	detG <- (m+1)^((m-1)/2)/m^{m/2}
	rhoc <- sqrt((m+2)/12)
	Omega <- pi^(m/2)/gamma(m/2+1)
	Theta <- Omega*rhoc^m/detG
	l <- (n1*detG)^(1/m)

	ballr <- l/2 * sqrt(m) + sqrt(sum(epsilon^2)) 
	ballr2 <- ballr^2
	N <- Omega * ballr^m / detG *1.1 + 1000*m
	
	FE <- matrix(0,N,m)
	nrowFE <- 1
	srowFE <- 1
	while(srowFE<=nrowFE ) {  
		tF <- FE[srowFE,1:m]
		maxF <- max(tF)
		minF <- min(tF)
		for(s in 1:m) if(tF[s]>=max(tF)-1) {
			tryF <- tF
			tryF[s] <- tryF[s]+1
			if(addfrom(m,tryF,ballr2)==s) { 
				nrowFE <- nrowFE +1
				FE[nrowFE,1:m] <- tryF
			}
		}
		for(s in 1:m) if(tF[s]<=min(tF)+1) {
			tryF <- tF
			tryF[s] <- tryF[s]-1
			if(addfrom(m,tryF,ballr2)==m+s) { 
				nrowFE <- nrowFE +1
				FE[nrowFE,1:m] <- tryF
			}
		}
		tryF <- tF+1
		if(addfrom(m,tryF,ballr2)==2*m+1) { 
			nrowFE <- nrowFE +1
			FE[nrowFE,1:m] <- tryF
		}
		tryF <- tF-1
		if(addfrom(m,tryF,ballr2)==2*m+2) { 
			nrowFE <- nrowFE +1
			FE[nrowFE,1:m] <- tryF
		}
		srowFE <- srowFE +1
		if(nrowFE>=dim(FE)[1]-2*m-2) FE=rbind(FE,matrix(0,N,m))
	}
	FE <- FE[1:nrowFE,]
		
	GR <- G %*% Ropt
	E <- FE %*% GR 
	candidates <- cbind(E,FE,matrix(1,dim(FE)[1],4))
	for(j in 1:m) { candidates <- candidates[ abs(candidates[,j]-epsilon[j])<l/2 ,]; }
	for(j in 1:m) candidates[,j] <- (candidates[,j]-epsilon[j]) / l + .5

	for(i in 1:dim(candidates)[1]) if(mod(sum(candidates[i,(m+1):(2*m)]),m+1)==0) candidates[i,2*m+1]=0; 
	
	list(candidates=candidates,design=Dopt,generator=G,rotation=Ropt,delta=-epsilonopt,Theta=Theta,l=l,worstdistance=rhoc/l,minfgr2=minfgr2)
}



SRSPD <- function(n, m=2, angle="magic", w=100){

	n <- as.integer(n)
	if(is.na(n)) stop("n must be an integer greater than one")
	if(n<=1) stop("n must be an integer greater than one")
	m <- as.integer(m)
	if(is.na(m)) stop("m must be an integer greater than one")
	if(m<=1) stop("m must be an integer greater than one")
	w <- as.integer(w)
	if(is.na(w)) stop("w must be an integer greater than one")
	if(w<=0) stop("w must be an positive integer")

	G <- matrix(-1,m,m)
	diag(G) <- m-sqrt(m+1)
	G <- G/(sqrt(m+1)-1)/sqrt(m)
	etamin <- sqrt(m+1)/sqrt(2*m)
	detG <- (m+1)^((m-1)/2)/m^{m/2}
	rhoc <- sqrt((m+2)/12)
	Omega <- pi^(m/2)/gamma(m/2+1)
	Theta <- Omega*rhoc^m/detG
	l <- (n*detG)^(1/m)
	
	nsd <- 0
	for(r in (m+1)/m^seq(1.7,.99,-0.05)) {
		ballr <- l/2 * sqrt(m) + rhoc*r 
		ballr2 <- ballr^2
		N <- Omega * ballr^m / detG *1.1 + 1000*m
		
		FE <- matrix(0,N,m)
		nrowFE <- 1
		srowFE <- 1
		while(srowFE<=nrowFE ) {  
			tF <- FE[srowFE,1:m]
			maxF <- max(tF)
			minF <- min(tF)
			for(s in 1:m) if(tF[s]>=max(tF)-1) {
				tryF <- tF
				tryF[s] <- tryF[s]+1
				if(addfrom(m,tryF,ballr2)==s) { 
					nrowFE <- nrowFE +1
					FE[nrowFE,1:m] <- tryF
				}
			}
			for(s in 1:m) if(tF[s]<=min(tF)+1) {
				tryF <- tF
				tryF[s] <- tryF[s]-1
				if(addfrom(m,tryF,ballr2)==m+s) { 
					nrowFE <- nrowFE +1
					FE[nrowFE,1:m] <- tryF
				}
			}
			tryF <- tF+1
			if(addfrom(m,tryF,ballr2)==2*m+1) { 
				nrowFE <- nrowFE +1
				FE[nrowFE,1:m] <- tryF
			}
			tryF <- tF-1
			if(addfrom(m,tryF,ballr2)==2*m+2) { 
				nrowFE <- nrowFE +1
				FE[nrowFE,1:m] <- tryF
			}
			srowFE <- srowFE +1
			if(nrowFE>=dim(FE)[1]-2*m-2) FE=rbind(FE,matrix(0,N,m))
		}
		FE <- FE[1:nrowFE,]
		
		minfgr2 <- 0  
		for(ll in 1:(w*2)) {
			
			CPair <- matrix(0,m*(m-1)/2,2)
			row <- 1
			for(i in 1:(m-1)) for(j in (i+1):m) { CPair[row,1] <- i; CPair[row,2] <- j; row <- row+1; }
			R <- diag(m)
			if(m>2|angle!='magic') for(a in 1:((m*(m-1)/2))) {
				alpha <- runif(1,0,2*pi)
				thepair <- CPair[a,]
				W <- diag(m)
				W[thepair[1],thepair[1]] <- cos(alpha); W[thepair[2],thepair[2]] <- cos(alpha); 
				W[thepair[1],thepair[2]] <- sin(alpha); W[thepair[2],thepair[1]] <- -sin(alpha); 
				R <- R%*%W
			}
			GR <- G %*% R
			E <- FE %*% GR 

			isE <- FALSE
			isB <- FALSE
			isL <- FALSE
			epsilonV <- matrix(0,3,m)
			for(i in 1:20) {
				epsilon <- runif(m,-rhoc*r,rhoc*r)
				D <- E
				for(j in 1:m) { if(length(D)<=m) break; D <- D[ abs(D[,j]-epsilon[j])<l/2 ,]; }
				if(length(D)<=m) next; 
				if(dim(D)[1]==n) { isE <- TRUE; epsilonV[1,] <- epsilon; break; }
				if(isL==FALSE & dim(D)[1]<n) { isL <- TRUE; epsilonV[2,] <- epsilon; }
				if(isB==FALSE & dim(D)[1]>n) { isB <- TRUE; epsilonV[3,] <- epsilon; }
				if(isL==TRUE & isB==TRUE) break;
			}
			try1 <- i
			if(try1==20) next; 
			try2 <- 0
			try3 <- 0
			
			if(isE==FALSE) for(j in 1:(m-1)) {
				try2 <- j
				epsilon <- c(epsilonV[2,1:j],epsilonV[3,(j+1):m])
				D <- E
				for(j in 1:m) { if(length(D)<=m) break;  D <- D[ abs(D[,j]-epsilon[j])<l/2 ,]; }
				if(length(D)<=m) next; 
				if(dim(D)[1]==n) { isE <- TRUE; epsilonV[1,] <- epsilon; break; }
				if(dim(D)[1]<n) { epsilonV[2,] <- epsilon; break; }
				if(dim(D)[1]>n) { epsilonV[3,] <- epsilon; }
			}

			if(isE==FALSE) for(k in 1:20) { 
				try3 <- k
				epsilon <- (epsilonV[2,]+epsilonV[3,])/2
				D <- E
				for(j in 1:m) { if(length(D)<=m) break; D <- D[ abs(D[,j]-epsilon[j])<l/2 ,]; }
				if(length(D)<=m) next; 
				if(dim(D)[1]==n) { isE <- TRUE; epsilonV[1,] <- epsilon; break; }
				if(dim(D)[1]<n) { epsilonV[2,] <- epsilon; }
				if(dim(D)[1]>n) { epsilonV[3,] <- epsilon; }
			}
			if(try3==20) next;
			
			nsd <- nsd+1
			for(j in 1:m) D[,j] <- (D[,j]-epsilon[j]) / l + .5

			fgr2D <- fgr2(D)
			if(nsd==1|fgr2D<minfgr2) {
				Dopt <- D
				minfgr2 <- fgr2D
				Ropt <- R
				epsilonopt <- epsilon
			}
			
			if(nsd>=w) break; 
		}
		
		if(nsd>=w) break; 
	}
		
	D <- Dopt
	DD = D 
	for(j in 1:m) DD[,j] <- (D[,j]- .5)*l +epsilonopt[j]
	DDD = round( DD %*% solve(Ropt) %*% solve(G) )
	slices = DDD[,1] 
	for(i in 1:dim(DDD)[1]) slices[i] = mod(sum(DDD[i,]),m+1)

	list(design=Dopt,slices=slices,generator=G,rotation=Ropt,delta=-epsilonopt,Theta=Theta,l=l,worstdistance=rhoc/l,minfgr2=minfgr2)
}

