source("SRSPD20170504.R")
library(randtoolbox)
library(DiceDesign)
library(RobustGaSP)

p=2
n1=13
n2=26
seed=1
ntrial=1000

f=function(x)
{
  f1=3/4*exp(-.25*(9*x[1]-2)^2-.25*(9*x[2]-2)^2)
  f2=3/4*exp(-1/49*(9*x[1]+1)^2-1/10*(9*x[2]+1)^2)
  f3=1/2*exp(-.25*(9*x[1]-7)^2-.25*(9*x[2]-3)^2)
  f4=1/5*exp(-(9*x[1]-4)^2-(9*x[2]-7)^2)
  val=f1+f2+f3-f4
  return(val)
}

	
PEns = function(design) {
	Y = rep(0,dim(design)[1])
	for(i in 1:dim(design)[1]) Y[i] = f(design[i,1:2])
	A = rgasp(design=design, response=Y)
	set.seed(1)
	testsample = cbind(runif(10000,0,1),runif(10000,0,1))
	realY = rep(0,10000)
	for(i in 1:10000) realY[i] = f(testsample[i,1:2])
	pre = predict(object=A,testing_input=testsample)$mean
	
	mean((pre-realY)^2)
}

sumDist = matrix(0,40,7)
sumPEns = matrix(0,40,7)
sumDist2 = matrix(0,40,7)
sumPEns2 = matrix(0,40,7)

# CVa, CVc, CVp, MMLH


for(seed in 1:ntrial){


resDist = matrix(0,40,7)
resPEns = matrix(0,40,7)


## MmLH-CV

set.seed(seed)
Dii = lhsDesign(n1,p)$design
Di = maximinSA_LHS(Dii)$design
D = Di
resp = rep(0,n1)
for(i in 1:n1) resp[i] = f(D[i,1:2])
supp = cbind(runif(5000,0,1),runif(5000,0,1))

resDist[n1,1] = min(dist(D))
resPEns[n1,1] = PEns(D)

for(n2 in 1:26){  

n = dim(D)[1]
preds = matrix(0,5000,n+1)
gp = rgasp(design=D, response=resp)
preds[,n+1]=predict(gp,supp)$mean
for(j in 1:n){ 
	gp = rgasp(design=D[-j,], response=resp[-j])
	preds[,j]=predict(gp,supp)$mean
}
scores = rep(0,5000)
for(j in 1:n) scores = scores + (preds[,j]-preds[,n+1])^2
scores = sqrt(scores/n)
for(i in 1:5000) {
	dd = (supp[i,1]-D[,1])^2
	dd = dd + (supp[i,2]-D[,2])^2
	scores[i] = scores[i] * sqrt(min(dd))
}

addindex = order(-scores)[1]
D = rbind(D,supp[addindex,])
resp = c(resp,f(supp[addindex,]))

resDist[n1+n2,1] = min(dist(D))
resPEns[n1+n2,1] = PEns(D)
}


## SRSPD-CV

set.seed(seed)
cans = RSPD_AmAms(n1,p,100)$candidates
inis = cans[cans[,2*p+1]==0,]
supp = cans[cans[,2*p+1]==1,1:2]
D = inis[,1:2]
resp = rep(0,n1)
for(i in 1:n1) resp[i] = f(D[i,1:2])

resDist[n1,2] = min(dist(D))
resPEns[n1,2] = PEns(D)

for(n2 in 1:26) { 
	if(dim(supp)[1]>=n2) {  

	n = dim(D)[1]
	preds = matrix(0,dim(supp)[1],n+1)
	gp = rgasp(design=D, response=resp)
	preds[,n+1]=predict(gp,supp)$mean
	for(j in 1:n){ 
		gp = rgasp(design=D[-j,], response=resp[-j])
		preds[,j]=predict(gp,supp)$mean
	}
	scores = rep(0,dim(supp)[1])
	for(j in 1:n) scores = scores + (preds[,j]-preds[,n+1])^2
	scores = sqrt(scores/n)
	for(i in 1:dim(supp)[1]) {
		dd = (supp[i,1]-D[,1])^2
		dd = dd + (supp[i,2]-D[,2])^2
		scores[i] = scores[i] * sqrt(min(dd))
	}

	addindex = order(-scores)[1]
	D = rbind(D,supp[addindex,])
	resp = c(resp,f(supp[addindex,]))
	}

resDist[n1+n2,2] = min(dist(D))
resPEns[n1+n2,2] = PEns(D)
}


## SRSPD-CV2

set.seed(seed)
cans = RSPD_AmAms(n1,p,100)$candidates
inis = cans[cans[,2*p+1]==0,]
D = inis[,1:2]
resp = rep(0,n1)
for(i in 1:n1) resp[i] = f(D[i,1:2])
supp = cans[cans[,2*p+1]==1,]

preds = matrix(0,dim(inis)[1],n1+1)
for(j in 1:n1){ 
	gp = rgasp(design=D[-j,], response=resp[-j])
	preds[,j]=predict(gp,inis)$mean
}
scores = rep(0,dim(inis)[1])
for(j in 1:n1) scores = scores + (preds[,j]-resp)^2
scores = sqrt(scores/n1)

means = rep(max(scores),dim(supp)[1])
for(j in 1:dim(supp)[1]) {
	isparent = rep(0,dim(inis)[1])
	for(i in 1:dim(inis)[1]) {
		if(max(inis[i,(p+1):(p+2)]-supp[j,(p+1):(p+2)])==1) if(min(inis[i,(p+1):(p+2)]-supp[j,(p+1):(p+2)])>=0) isparent[i]=1
		if(max(inis[i,(p+1):(p+2)]-supp[j,(p+1):(p+2)])<=0) if(min(inis[i,(p+1):(p+2)]-supp[j,(p+1):(p+2)])==-1) isparent[i]=1
	}
	if(sum(isparent)>0) means[j]=mean(scores[isparent==1])
}

#plot(D,xlim=c(0,1),ylim=c(0,1))
#text(x=D[,1],y=D[,2],floor(scores*1000))
#text(x=supp[,1],y=supp[,2],floor(means*1000),col='red')


resDist[n1,3] = min(dist(D))
resPEns[n1,3] = PEns(D)

for(n2 in 1:26) { 
	if(n2<=dim(supp)[1]) D = rbind(inis,supp[order(-means)[1:n2],])[,1:p]
	if(n2>dim(supp)[1]) D = rbind(inis,supp)[,1:p]
	if(n2==0) D = inis[,1:p]

	resDist[n1+n2,3] = min(dist(D))
	resPEns[n1+n2,3] = PEns(D)
}

resPEns[1,4]=1
sumDist=sumDist+resDist
sumPEns=sumPEns+resPEns
sumDist2=sumDist2+resDist^2
sumPEns2=sumPEns2+resPEns^2
print('')
print('')
print(seed)


write.table(x=sumDist,file=paste('simu3CFrankeressumDist',p,'.txt',sep=''),quote=FALSE,row.names=FALSE,col.names=FALSE)
write.table(x=sumPEns,file=paste('simu3CFrankeressumPEns',p,'.txt',sep=''),quote=FALSE,row.names=FALSE,col.names=FALSE)
write.table(x=sumDist,file=paste('simu3CFrankeressumDist2',p,'.txt',sep=''),quote=FALSE,row.names=FALSE,col.names=FALSE)
write.table(x=sumPEns,file=paste('simu3CFrankeressumPEns2',p,'.txt',sep=''),quote=FALSE,row.names=FALSE,col.names=FALSE)
}

#resDist=sumDist/ntrial
#resPEns=sumPEns/ntrial
#
#write.table(x=resDist,file=paste('simu3CFrankeresDist',p,'.txt',sep=''),quote=FALSE,row.names=FALSE,col.names=FALSE)
#write.table(x=resPEns,file=paste('simu3CFrankeresPEns',p,'.txt',sep=''),quote=FALSE,row.names=FALSE,col.names=FALSE)
